@@ -53,7 +53,8 @@ struct tslib_iir {
 };
 
 static void iir_filter(struct tslib_module_info *info, int32_t *new,
-		       int32_t *save) {
+		       int32_t *save)
+{
 	struct tslib_iir *iir = (struct tslib_iir *)info;
 
 	*save = (iir->N * *save + (iir->D - iir->N) * *new + iir->D / 2) / iir->D;
@@ -141,7 +142,7 @@ static int iir_read_mt(struct tslib_module_info *info,
 
 	for (i = 0; i < ret; i++) {
 		for (j = 0; j < max_slots; j++) {
-			if (samp[i][j].valid != 1)
+			if (!(samp[i][j].valid & TSLIB_MT_VALID))
 				continue;
 			/* reset */
 			if (samp[i][j].pressure == 0) { /* pen-up */
