@@ -111,8 +111,8 @@ struct tslib_input {
 	int	current_y;
 	int	current_p;
 
-	int	using_syn;
-	int	grab_events;
+	int8_t	using_syn;
+	int8_t	grab_events;
 
 	struct input_event ev[NUM_EVENTS_READ];
 	struct ts_sample_mt **buf;
@@ -125,6 +125,7 @@ struct tslib_input {
 	int8_t	mt;
 	int8_t	no_pressure;
 	int8_t	type_a;
+	int32_t *last_pressure;
 
 	uint16_t	special_device; /* broken device we work around, see below */
 };
@@ -233,10 +234,9 @@ static int check_fd(struct tslib_input *i)
 			"tslib: Warning: Selected device uses a different version of the event protocol than tslib was compiled for\n");
 
 	if ((ioctl(ts->fd, EVIOCGBIT(0, sizeof(evbit)), evbit) < 0) ||
-		!(evbit[BIT_WORD(EV_ABS)] & BIT_MASK(EV_ABS)) ||
-		!(evbit[BIT_WORD(EV_KEY)] & BIT_MASK(EV_KEY))) {
+		!(evbit[BIT_WORD(EV_ABS)] & BIT_MASK(EV_ABS))) {
 		fprintf(stderr,
-			"tslib: Selected device is not a touchscreen (must support ABS and KEY event types)\n");
+			"tslib: Selected device is not a touchscreen (must support ABS event type)\n");
 		return -1;
 	}
 
@@ -251,18 +251,6 @@ static int check_fd(struct tslib_input *i)
 		}
 	}
 
-	if (ioctl(ts->fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit) < 0) {
-		fprintf(stderr, "tslib: ioctl EVIOCGBIT error)\n");
-		return -1;
-	}
-
-
-	if (evbit[BIT_WORD(EV_SYN)] & BIT_MASK(EV_SYN))
-		i->using_syn = 1;
-
-	/* read device info and set special device nr */
-	get_special_device(i);
-
 	/* Remember whether we have a multitouch device. We need to know for ABS_X,
 	 * ABS_Y and ABS_PRESSURE data.
 	 */
@@ -270,6 +258,30 @@ static int check_fd(struct tslib_input *i)
 	    (absbit[BIT_WORD(ABS_MT_POSITION_Y)] & BIT_MASK(ABS_MT_POSITION_Y)))
 		i->mt = 1;
 
+	if (evbit[BIT_WORD(EV_KEY)] & BIT_MASK(EV_KEY)) {
+		if (ioctl(ts->fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit) < 0) {
+			fprintf(stderr, "tslib: ioctl EVIOCGBIT error)\n");
+			return -1;
+		}
+
+		/* for multitouch type B, tracking id is enough for pen down/up. type A
+		 * has pen down/up through the list of (empty) SYN_MT_REPORT
+		 * only for singletouch we need BTN_TOUCH or BTN_LEFT
+		 */
+		if (!(keybit[BIT_WORD(BTN_TOUCH)] & BIT_MASK(BTN_TOUCH) ||
+		      keybit[BIT_WORD(BTN_LEFT)] & BIT_MASK(BTN_LEFT)) && !i->mt) {
+			fprintf(stderr,
+				"tslib: Selected device is not a touchscreen (missing BTN_TOUCH or BTN_LEFT)\n");
+			return -1;
+		}
+	}
+
+	if (evbit[BIT_WORD(EV_SYN)] & BIT_MASK(EV_SYN))
+		i->using_syn = 1;
+
+	/* read device info and set special device nr */
+	get_special_device(i);
+
 	/* Since some touchscreens (eg. infrared) physically can't measure pressure,
 	 * the input system doesn't report it on those. Tslib relies on pressure, thus
 	 * we set it to constant 255. It's still controlled by BTN_TOUCH/BTN_LEFT -
@@ -290,18 +302,10 @@ static int check_fd(struct tslib_input *i)
 
 	/* remember whether we have a multitouch type A device */
 	if (i->mt && synbit[BIT_WORD(SYN_MT_REPORT)] & BIT_MASK(SYN_MT_REPORT) &&
-	    !(absbit[BIT_WORD(ABS_MT_SLOT)] & BIT_MASK(ABS_MT_SLOT)))
+	    !(absbit[BIT_WORD(ABS_MT_SLOT)] & BIT_MASK(ABS_MT_SLOT)) &&
+	    !(absbit[BIT_WORD(ABS_MT_TRACKING_ID)] & BIT_MASK(ABS_MT_TRACKING_ID)))
 		i->type_a = 1;
 
-	/* for multitouch type B, tracking id is enough for pen down/up. type A
-	 * has pen down/up through the list of (empty) SYN_MT_REPORT */
-	if (!(keybit[BIT_WORD(BTN_TOUCH)] & BIT_MASK(BTN_TOUCH) ||
-	      keybit[BIT_WORD(BTN_LEFT)] & BIT_MASK(BTN_LEFT)) && i->mt != 1) {
-		fprintf(stderr,
-			"tslib: Selected device is not a touchscreen (missing BTN_TOUCH or BTN_LEFT)\n");
-		return -1;
-	}
-
 	if (i->grab_events == GRAB_EVENTS_WANTED) {
 		if (ioctl(ts->fd, EVIOCGRAB, (void *)1)) {
 			fprintf(stderr,
@@ -311,6 +315,12 @@ static int check_fd(struct tslib_input *i)
 		i->grab_events = GRAB_EVENTS_ACTIVE;
 	}
 
+	if (i->mt && !i->using_syn) {
+		fprintf(stderr,
+			"tslib: Unsupported multitouch device (missing EV_SYN)\n");
+		return -1;
+	}
+
 	return ts->fd;
 }
 
@@ -441,6 +451,13 @@ static int ts_input_read(struct tslib_module_info *inf,
 					case ABS_PRESSURE:
 						i->current_p = ev.value;
 						break;
+					case ABS_MT_PRESSURE:
+						i->current_p = ev.value;
+						break;
+					case ABS_MT_TRACKING_ID:
+						if (ev.value == -1)
+							i->current_p = 0;
+						break;
 					}
 				}
 				break;
@@ -547,6 +564,13 @@ static int ts_input_read_mt(struct tslib_module_info *inf,
 	int rd;
 	int j, k;
 	uint8_t pen_up = 0;
+	static int32_t next_trackid;
+
+	if (ts->fd != i->last_fd)
+		i->last_fd = check_fd(i);
+
+	if (i->last_fd == -1)
+		return -ENODEV;
 
 	if (i->buf == NULL || i->max_slots < max_slots || i->nr < nr) {
 		if (i->buf) {
@@ -570,14 +594,18 @@ static int ts_input_read_mt(struct tslib_module_info *inf,
 
 		i->max_slots = max_slots;
 		i->nr = nr;
+
+		if (i->type_a) {
+			i->last_pressure = calloc(max_slots, sizeof(int32_t));
+			if (!i->last_pressure) {
+				for (j = 0; j < nr; j++)
+					free(i->buf[j]);
+
+				return -ENOMEM;
+			}
+		}
 	}
 
-	if (ts->fd != i->last_fd)
-		i->last_fd = check_fd(i);
-
-	if (i->last_fd == -1)
-		return -ENODEV;
-
 	if (i->no_pressure)
 		set_pressure(i);
 
@@ -588,328 +616,263 @@ static int ts_input_read_mt(struct tslib_module_info *inf,
 		}
 	}
 
-	if (i->using_syn) {
-		while (total < nr) {
-			memset(i->ev, 0, sizeof(i->ev));
-			rd = read(ts->fd,
-				  i->ev,
-				  sizeof(struct input_event) * NUM_EVENTS_READ);
-			if (rd == -1) {
-				if (total == 0) {
-					if (errno > 0)
-						return errno * -1;
-					else
-						return errno;
-				} else {
-					return total;
-				}
-			} else if (rd < (int) sizeof(struct input_event)) {
-				if (total == 0)
-					return -1;
+	while (total < nr) {
+		memset(i->ev, 0, sizeof(i->ev));
+		rd = read(ts->fd,
+			  i->ev,
+			  sizeof(struct input_event) * NUM_EVENTS_READ);
+		if (rd == -1) {
+			if (total == 0) {
+				if (errno > 0)
+					return errno * -1;
 				else
-					return total;
+					return errno;
+			} else {
+				return total;
 			}
-
-			for (it = 0; it < rd / sizeof(struct input_event); it++) {
-			#ifdef DEBUG
-				printf("INPUT-RAW: read type %d  code %3d  value %4d  time %ld.%ld\n",
-				       i->ev[it].type, i->ev[it].code,
-				       i->ev[it].value, (long)i->ev[it].time.tv_sec,
-				       (long)i->ev[it].time.tv_usec);
-			#endif
-				switch (i->ev[it].type) {
-				case EV_KEY:
-					switch (i->ev[it].code) {
-					case BTN_TOUCH:
-					case BTN_LEFT:
-						if (i->ev[it].code == BTN_TOUCH) {
-							i->buf[total][i->slot].pen_down = i->ev[it].value;
-							i->buf[total][i->slot].tv = i->ev[it].time;
-							if (i->ev[it].value == 0)
-								pen_up = 1;
-						}
-						break;
-					}
-					break;
-				case EV_SYN:
-					switch (i->ev[it].code) {
-					case SYN_REPORT:
-						if (pen_up && i->no_pressure) {
-							for (k = 0; k < max_slots; k++) {
-								if (i->buf[total][k].x != 0 ||
-								    i->buf[total][k].y != 0 ||
-								    i->buf[total][k].pressure != 0)
-									i->buf[total][i->slot].valid = 1;
-
-								i->buf[total][k].x = 0;
-								i->buf[total][k].y = 0;
-								i->buf[total][k].pressure = 0;
-							}
-
-						if (pen_up)
-							pen_up = 0;
-						}
-
-						for (j = 0; j < nr; j++) {
-							for (k = 0; k < max_slots; k++) {
-								memcpy(&samp[j][k],
-									&i->buf[j][k],
-									sizeof(struct ts_sample_mt));
-							}
-						}
-
-						if (i->type_a)
-							i->slot = 0;
-
-						total++;
-						break;
-					case SYN_MT_REPORT:
-						if (!i->type_a)
-							break;
-
-						if (i->buf[total][i->slot].valid != 1) {
-							i->buf[total][i->slot].pressure = 0;
-							if (i->slot == 0) {
-								pen_up = 1;
-								break;
-							}
-
-						} else if (i->slot < (max_slots - 1)) {
-							i->slot++;
-							i->buf[total][i->slot].slot = i->slot;
-						}
-
-						break;
-				#ifdef DEBUG
-					case SYN_DROPPED:
-						fprintf(stderr,
-							"INPUT-RAW: SYN_DROPPED\n");
-						break;
-				#endif
-					}
-					break;
-				case EV_ABS:
-					switch (i->ev[it].code) {
-					case ABS_X:
-						/* in case we didn't already get data for this
-						 * slot, we go ahead and act as if this would be
-						 * ABS_MT_POSITION_X
-						 */
-						if (i->mt && i->buf[total][i->slot].valid == 1)
-							break;
-					case ABS_MT_POSITION_X:
-						i->buf[total][i->slot].x = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_Y:
-						if (i->mt && i->buf[total][i->slot].valid == 1)
-							break;
-					case ABS_MT_POSITION_Y:
-						i->buf[total][i->slot].y = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_PRESSURE:
-						if (i->mt && i->buf[total][i->slot].valid == 1)
-							break;
-					case ABS_MT_PRESSURE:
-						i->buf[total][i->slot].pressure = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TOOL_X:
-						i->buf[total][i->slot].tool_x = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TOOL_Y:
-						i->buf[total][i->slot].tool_y = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TOOL_TYPE:
-						i->buf[total][i->slot].tool_type = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_ORIENTATION:
-						i->buf[total][i->slot].orientation = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_DISTANCE:
-						i->buf[total][i->slot].distance = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-
-						if (i->special_device == EGALAX_VERSION_210) {
-							if (i->ev[it].value > 0)
-								i->buf[total][i->slot].pressure = 0;
-							else
-								i->buf[total][i->slot].pressure = 255;
-						}
-
-						break;
-					case ABS_MT_BLOB_ID:
-						i->buf[total][i->slot].blob_id = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TOUCH_MAJOR:
-						i->buf[total][i->slot].touch_major = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_WIDTH_MAJOR:
-						i->buf[total][i->slot].width_major = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TOUCH_MINOR:
-						i->buf[total][i->slot].touch_minor = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_WIDTH_MINOR:
-						i->buf[total][i->slot].width_minor = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						break;
-					case ABS_MT_TRACKING_ID:
-						i->buf[total][i->slot].tracking_id = i->ev[it].value;
-						i->buf[total][i->slot].tv = i->ev[it].time;
-						i->buf[total][i->slot].valid = 1;
-						if (i->ev[it].value == -1)
-							i->buf[total][i->slot].pressure = 0;
-						break;
-					case ABS_MT_SLOT:
-						if (i->ev[it].value < 0 || i->ev[it].value >= max_slots) {
-							fprintf(stderr, "tslib: warning: slot out of range. data corrupted!\n");
-							i->slot = max_slots - 1;
-						} else {
-							i->slot = i->ev[it].value;
-							i->buf[total][i->slot].slot = i->ev[it].value;
-							i->buf[total][i->slot].valid = 1;
-						}
-						break;
-					case ABS_X+2:
-						if (i->special_device == EGALAX_VERSION_112) {
-							/* this is ABS_Z wrongly used as ABS_X here */
-							if (i->mt && i->buf[total][i->slot].valid == 1)
-								break;
-
-							i->buf[total][i->slot].x = i->ev[it].value;
-							i->buf[total][i->slot].tv = i->ev[it].time;
-							i->buf[total][i->slot].valid = 1;
-						}
-						break;
-					case ABS_Y+2:
-						if (i->special_device == EGALAX_VERSION_112) {
-							/* this is ABS_RX wrongly used as ABS_Y here */
-							if (i->mt && i->buf[total][i->slot].valid == 1)
-								break;
-
-							i->buf[total][i->slot].y = i->ev[it].value;
-							i->buf[total][i->slot].tv = i->ev[it].time;
-							i->buf[total][i->slot].valid = 1;
-						}
-						break;
-					}
-					break;
-				}
-				if (total == ret)
-					break;
-			} /* just NUM_EVENTS_READ. it's simply 1 */
+		} else if (rd < (int) sizeof(struct input_event)) {
+			if (total == 0)
+				return -1;
+			else
+				return total;
 		}
-		ret = total;
-	} else {
-		/*
-		 * XXX This could be simplified because it duplicates the above ts_read()
-		 * code for ts_read_mt() to stay compatible and also support single touch
-		 * protocols without EV_SYN.
-		 */
-		struct input_event ev_single;
-		unsigned char *p = (unsigned char *) &ev_single;
-		int len = sizeof(struct input_event);
 
-		while (total < nr) {
-			ret = read(ts->fd, p, len);
-			if (ret == -1) {
-				if (errno == EINTR)
-					continue;
-
-				break;
-			}
-
-			if (ret < (int)sizeof(struct input_event)) {
-				/* short read
-				 * restart read to get the rest of the event
-				 */
-				p += ret;
-				len -= ret;
-				continue;
-			}
-			/* successful read of a whole event */
-
-			if (ev_single.type == EV_ABS) {
-				switch (ev_single.code) {
-				case ABS_X:
-					if (ev_single.value != 0) {
-						samp[total][0].x = i->current_x = ev_single.value;
-						samp[total][0].y = i->current_y;
-						samp[total][0].pressure = i->current_p;
-					} else {
-						fprintf(stderr, "tslib: dropped x = 0\n");
-						continue;
-					}
-					break;
-				case ABS_Y:
-					if (ev_single.value != 0) {
-						samp[total][0].x = i->current_x;
-						samp[total][0].y = i->current_y = ev_single.value;
-						samp[total][0].pressure = i->current_p;
-					} else {
-						fprintf(stderr, "tslib: dropped y = 0\n");
-						continue;
-					}
-					break;
-				case ABS_PRESSURE:
-					samp[total][0].x = i->current_x;
-					samp[total][0].y = i->current_y;
-					samp[total][0].pressure = i->current_p = ev_single.value;
-					break;
-				}
-				samp[total][0].tv = ev_single.time;
-	#ifdef DEBUG
-				fprintf(stderr, "RAW---------------------------> %d %d %d\n",
-					samp[total][0].x, samp[total][0].y, samp[total][0].pressure);
-	#endif /* DEBUG */
-				samp++;
-				total++;
-			} else if (ev_single.type == EV_KEY) {
-				switch (ev_single.code) {
+		for (it = 0; it < rd / sizeof(struct input_event); it++) {
+		#ifdef DEBUG
+			printf("INPUT-RAW: read type %d  code %3d  value %4d  time %ld.%ld\n",
+			       i->ev[it].type, i->ev[it].code,
+			       i->ev[it].value, (long)i->ev[it].time.tv_sec,
+			       (long)i->ev[it].time.tv_usec);
+		#endif
+			switch (i->ev[it].type) {
+			case EV_KEY:
+				switch (i->ev[it].code) {
 				case BTN_TOUCH:
 				case BTN_LEFT:
-					if (ev_single.value == 0) {
-						/* pen up */
-						samp[total][0].x = 0;
-						samp[total][0].y = 0;
-						samp[total][0].pressure = 0;
-						samp[total][0].tv = ev_single.time;
-						total++;
+					if (i->ev[it].code == BTN_TOUCH) {
+						i->buf[total][i->slot].pen_down = i->ev[it].value;
+						i->buf[total][i->slot].tv = i->ev[it].time;
+						if (i->ev[it].value == 0)
+							pen_up = 1;
 					}
 					break;
 				}
-			} else {
-				fprintf(stderr, "tslib: Unknown event type %d\n", ev_single.type);
+				break;
+			case EV_SYN:
+				switch (i->ev[it].code) {
+				case SYN_REPORT:
+					if (pen_up && i->no_pressure) {
+						for (k = 0; k < max_slots; k++) {
+							if (i->buf[total][k].x != 0 ||
+							    i->buf[total][k].y != 0 ||
+							    i->buf[total][k].pressure != 0)
+								i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+
+							i->buf[total][k].x = 0;
+							i->buf[total][k].y = 0;
+							i->buf[total][k].pressure = 0;
+						}
+					}
+
+					if (pen_up)
+						pen_up = 0;
+
+					for (j = 0; j < nr; j++) {
+						for (k = 0; k < max_slots; k++) {
+							/* generate tracking id if not available */
+							if (i->type_a) {
+								if (i->buf[j][k].pressure == 0) {
+									i->buf[j][k].tracking_id = -1;
+									i->last_pressure[k] = 0;
+								} else {
+									if (i->last_pressure[k] == 0)
+										next_trackid++;
+
+									i->buf[j][k].tracking_id = next_trackid;
+								}
+							}
+
+							memcpy(&samp[j][k],
+								&i->buf[j][k],
+								sizeof(struct ts_sample_mt));
+						}
+					}
+
+					if (i->type_a)
+						i->slot = 0;
+
+					total++;
+					break;
+				case SYN_MT_REPORT:
+					if (!i->type_a)
+						break;
+
+					if (i->buf[total][i->slot].valid < 1) {
+						i->buf[total][i->slot].pressure = 0;
+						if (i->slot == 0) {
+							pen_up = 1;
+							break;
+						}
+
+					} else if (i->slot < (max_slots - 1)) {
+						i->slot++;
+						i->buf[total][i->slot].slot = i->slot;
+					}
+
+					break;
+			#ifdef DEBUG
+				case SYN_DROPPED:
+					fprintf(stderr,
+						"INPUT-RAW: SYN_DROPPED\n");
+					break;
+			#endif
+				}
+				break;
+			case EV_ABS:
+				switch (i->ev[it].code) {
+				case ABS_X:
+					/* in case we didn't already get data for this
+					 * slot, we go ahead and act as if this would be
+					 * ABS_MT_POSITION_X
+					 */
+					if (i->mt && i->buf[total][i->slot].valid & TSLIB_MT_VALID)
+						break;
+				case ABS_MT_POSITION_X:
+					i->buf[total][i->slot].x = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_Y:
+					if (i->mt && i->buf[total][i->slot].valid & TSLIB_MT_VALID)
+						break;
+				case ABS_MT_POSITION_Y:
+					i->buf[total][i->slot].y = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_PRESSURE:
+					if (i->mt && i->buf[total][i->slot].valid & TSLIB_MT_VALID)
+						break;
+				case ABS_MT_PRESSURE:
+					i->buf[total][i->slot].pressure = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_TOOL_X:
+					i->buf[total][i->slot].tool_x = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					/* for future use
+					 * i->buf[total][i->slot].valid |= TSLIB_MT_VALID_TOOL;
+					 */
+					break;
+				case ABS_MT_TOOL_Y:
+					i->buf[total][i->slot].tool_y = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					/* for future use
+					 * i->buf[total][i->slot].valid |= TSLIB_MT_VALID_TOOL;
+					 */
+					break;
+				case ABS_MT_TOOL_TYPE:
+					i->buf[total][i->slot].tool_type = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					/* for future use
+					 * i->buf[total][i->slot].valid |= TSLIB_MT_VALID_TOOL;
+					 */
+					break;
+				case ABS_MT_ORIENTATION:
+					i->buf[total][i->slot].orientation = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_DISTANCE:
+					i->buf[total][i->slot].distance = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+
+					if (i->special_device == EGALAX_VERSION_210) {
+						if (i->ev[it].value > 0)
+							i->buf[total][i->slot].pressure = 0;
+						else
+							i->buf[total][i->slot].pressure = 255;
+					}
+
+					break;
+				case ABS_MT_BLOB_ID:
+					i->buf[total][i->slot].blob_id = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_TOUCH_MAJOR:
+					i->buf[total][i->slot].touch_major = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_WIDTH_MAJOR:
+					i->buf[total][i->slot].width_major = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_TOUCH_MINOR:
+					i->buf[total][i->slot].touch_minor = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_WIDTH_MINOR:
+					i->buf[total][i->slot].width_minor = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					break;
+				case ABS_MT_TRACKING_ID:
+					i->buf[total][i->slot].tracking_id = i->ev[it].value;
+					i->buf[total][i->slot].tv = i->ev[it].time;
+					i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					if (i->ev[it].value == -1)
+						i->buf[total][i->slot].pressure = 0;
+					break;
+				case ABS_MT_SLOT:
+					if (i->ev[it].value < 0 || i->ev[it].value >= max_slots) {
+						fprintf(stderr, "tslib: warning: slot out of range. data corrupted!\n");
+						i->slot = max_slots - 1;
+					} else {
+						i->slot = i->ev[it].value;
+						i->buf[total][i->slot].slot = i->ev[it].value;
+						i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					}
+					break;
+				case ABS_X+2:
+					if (i->special_device == EGALAX_VERSION_112) {
+						/* this is ABS_Z wrongly used as ABS_X here */
+						if (i->mt && i->buf[total][i->slot].valid & TSLIB_MT_VALID)
+							break;
+
+						i->buf[total][i->slot].x = i->ev[it].value;
+						i->buf[total][i->slot].tv = i->ev[it].time;
+						i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					}
+					break;
+				case ABS_Y+2:
+					if (i->special_device == EGALAX_VERSION_112) {
+						/* this is ABS_RX wrongly used as ABS_Y here */
+						if (i->mt && i->buf[total][i->slot].valid & TSLIB_MT_VALID)
+							break;
+
+						i->buf[total][i->slot].y = i->ev[it].value;
+						i->buf[total][i->slot].tv = i->ev[it].time;
+						i->buf[total][i->slot].valid |= TSLIB_MT_VALID;
+					}
+					break;
+				}
+				break;
 			}
-			p = (unsigned char *) &ev_single;
-		}
-		ret = total;
+			if (total == ret)
+				break;
+		} /* just NUM_EVENTS_READ. it's simply 1 */
 	}
 
-	return ret;
+	return total;
 }
 
 static int ts_input_fini(struct tslib_module_info *inf)
@@ -930,6 +893,9 @@ static int ts_input_fini(struct tslib_module_info *inf)
 	if (i->buf)
 		free(i->buf);
 
+	if (i->last_pressure)
+		free(i->last_pressure);
+
 	free(inf);
 
 	return 0;
@@ -995,6 +961,7 @@ TSAPI struct tslib_module_info *input_mod_init(__attribute__ ((unused)) struct t
 	i->last_fd = -2;
 	i->type_a = 0;
 	i->special_device = 0;
+	i->last_pressure = NULL;
 
 	if (tslib_parse_vars(&i->module, raw_vars, NR_VARS, params)) {
 		free(i);
