@@ -13,10 +13,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#ifdef HAVE_LIBDL
 #include <dlfcn.h>
+#endif
+
+#include <errno.h>
 
 #if !defined(HAVE_STRSEP)
 #include "ts_strsep.h"
@@ -58,7 +64,8 @@ int ts_config(struct tsdev *ts)
 		if (conffile) {
 			strdup_allocated = 1;
 		} else {
-			perror("Couldn't find tslib config file");
+			ts_error("Couldn't find tslib config file: %s\n",
+				 strerror(errno));
 			return -1;
 		}
 	}
@@ -68,7 +75,8 @@ int ts_config(struct tsdev *ts)
 		if (strdup_allocated)
 			free(conffile);
 
-		perror("Couldnt open tslib config file");
+		ts_error("Couldn't open tslib config file: %s\n",
+			 strerror(errno));
 		return -1;
 	}
 
@@ -158,9 +166,13 @@ int ts_reconfig(struct tsdev *ts)
 		next = info->next;
 
 		handle = info->handle;
-		info->ops->fini(info);
+
+		if (info->ops->fini)
+			info->ops->fini(info);
+	#ifdef HAVE_LIBDL
 		if (handle)
 			dlclose(handle);
+	#endif
 
 		info = next;
 	}
