@@ -11,8 +11,11 @@
  */
 
 #include "tslib.h"
+#include "tslib-private.h"
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
+#include <string.h>
 
 static const char * const ts_name_default[] = {
 		"/dev/input/ts",
@@ -44,7 +47,7 @@ struct tsdev *ts_setup(const char *dev_name, int nonblock)
 
 	/* if detected try to configure it */
 	if (ts && ts_config(ts) != 0) {
-		perror("ts_config");
+		ts_error("ts_config: %s\n", strerror(errno));
 		ts_close(ts);
 		return NULL;
 	}
