@@ -457,7 +457,7 @@ void fillrect (int32_t x1, int32_t y1, int32_t x2, int32_t y2, uint32_t colidx)
 
 	for (; y1 <= y2; y1++) {
 		for (tmp = x1; tmp <= x2; tmp++) {
-			__pixel_loc(x1, y1, &loc);
+			__pixel_loc(tmp, y1, &loc);
 			__setpixel(loc, xormode, colidx);
 			loc.p8 += bytes_per_pixel;
 		}
