@@ -7,7 +7,9 @@
  * COPYING for more details.
  *
  *
- * Basic test program for touchscreen library.
+ * Graphical touchscreen calibration tool. This writes the configuration
+ * file used by tslib's "linear" filter plugin module to transform the
+ * touch samples according to the calibration.
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,18 +27,13 @@
 
 #include "fbutils.h"
 #include "testutils.h"
+#include "ts_calibrate.h"
 
 static int palette[] = {
 	0x000000, 0xffe080, 0xffffff, 0xe0c0a0
 };
 #define NR_COLORS (sizeof(palette) / sizeof(palette[0]))
 
-typedef struct {
-	int x[5], xfb[5];
-	int y[5], yfb[5];
-	int a[7];
-} calibration;
-
 static void sig(int sig)
 {
 	close_framebuffer();
@@ -46,79 +43,6 @@ static void sig(int sig)
 	exit(1);
 }
 
-static int perform_calibration(calibration *cal)
-{
-	int j;
-	float n, x, y, x2, y2, xy, z, zx, zy;
-	float det, a, b, c, e, f, i;
-	float scaling = 65536.0;
-
-	/* Get sums for matrix */
-	n = x = y = x2 = y2 = xy = 0;
-	for (j = 0; j < 5; j++) {
-		n += 1.0;
-		x += (float)cal->x[j];
-		y += (float)cal->y[j];
-		x2 += (float)(cal->x[j]*cal->x[j]);
-		y2 += (float)(cal->y[j]*cal->y[j]);
-		xy += (float)(cal->x[j]*cal->y[j]);
-	}
-
-	/* Get determinant of matrix -- check if determinant is too small */
-	det = n*(x2*y2 - xy*xy) + x*(xy*y - x*y2) + y*(x*xy - y*x2);
-	if (det < 0.1 && det > -0.1) {
-		printf("ts_calibrate: determinant is too small -- %f\n", det);
-		return 0;
-	}
-
-	/* Get elements of inverse matrix */
-	a = (x2*y2 - xy*xy)/det;
-	b = (xy*y - x*y2)/det;
-	c = (x*xy - y*x2)/det;
-	e = (n*y2 - y*y)/det;
-	f = (x*y - n*xy)/det;
-	i = (n*x2 - x*x)/det;
-
-	/* Get sums for x calibration */
-	z = zx = zy = 0;
-	for (j = 0; j < 5; j++) {
-		z += (float)cal->xfb[j];
-		zx += (float)(cal->xfb[j]*cal->x[j]);
-		zy += (float)(cal->xfb[j]*cal->y[j]);
-	}
-
-	/* Now multiply out to get the calibration for framebuffer x coord */
-	cal->a[0] = (int)((a*z + b*zx + c*zy)*(scaling));
-	cal->a[1] = (int)((b*z + e*zx + f*zy)*(scaling));
-	cal->a[2] = (int)((c*z + f*zx + i*zy)*(scaling));
-
-	printf("%f %f %f\n", (a*z + b*zx + c*zy),
-			     (b*z + e*zx + f*zy),
-			     (c*z + f*zx + i*zy));
-
-	/* Get sums for y calibration */
-	z = zx = zy = 0;
-	for (j = 0; j < 5; j++) {
-		z += (float)cal->yfb[j];
-		zx += (float)(cal->yfb[j]*cal->x[j]);
-		zy += (float)(cal->yfb[j]*cal->y[j]);
-	}
-
-	/* Now multiply out to get the calibration for framebuffer y coord */
-	cal->a[3] = (int)((a*z + b*zx + c*zy)*(scaling));
-	cal->a[4] = (int)((b*z + e*zx + f*zy)*(scaling));
-	cal->a[5] = (int)((c*z + f*zx + i*zy)*(scaling));
-
-	printf("%f %f %f\n", (a*z + b*zx + c*zy),
-			     (b*z + e*zx + f*zy),
-			     (c*z + f*zx + i*zy));
-
-	/* If we got here, we're OK, so assign scaling to a[6] and return */
-	cal->a[6] = (int)scaling;
-
-	return 1;
-}
-
 static void get_sample(struct tsdev *ts, calibration *cal,
 		       int index, int x, int y, char *name)
 {
@@ -171,7 +95,7 @@ static void clearbuf(struct tsdev *ts)
 			break;
 
 		if (ts_read_raw(ts, &sample, 1) < 0) {
-			perror("ts_read");
+			perror("ts_read_raw");
 			exit(1);
 		}
 	}
@@ -181,7 +105,13 @@ static void help(void)
 {
 	struct ts_lib_version_data *ver = ts_libversion();
 
-	printf("tslib %s (library 0x%X)\n", ver->package_version, ver->version_num);
+	printf("                 _       _ _ _\n");
+	printf("                | |_ ___| (_) |__\n");
+	printf("                | __/ __| | | '_ \\\n");
+	printf("                | |_\\__ \\ | | |_) |\n");
+	printf("                 \\__|___/_|_|_.__/\n\n");
+	printf("tslib %s / libts ABI version %d (0x%06X)\n",
+		ver->package_version, ver->version_num >> 16, ver->version_num);
 	printf("\n");
 	printf("Usage: ts_calibrate [-r <rotate_value>]\n");
 	printf("\n");
@@ -243,7 +173,8 @@ int main(int argc, char **argv)
 		}
 
 		if (errno) {
-			char *str = "option ?";
+			char str[9];
+			sprintf(str, "option ?");
 			str[7] = c & 0xff;
 			perror(str);
 		}
