@@ -57,7 +57,13 @@ static void usage(char **argv)
 {
 	struct ts_lib_version_data *ver = ts_libversion();
 
-	printf("tslib %s (library 0x%X)\n", ver->package_version, ver->version_num);
+	printf("                 _       _ _ _\n");
+	printf("                | |_ ___| (_) |__\n");
+	printf("                | __/ __| | | '_ \\\n");
+	printf("                | |_\\__ \\ | | |_) |\n");
+	printf("                 \\__|___/_|_|_.__/\n\n");
+	printf("tslib %s / libts ABI version %d (0x%06X)\n",
+		ver->package_version, ver->version_num >> 16, ver->version_num);
 	printf("\n");
 	printf("Usage: %s [--raw] [--non-blocking] [-s samples] [-i <device>] [-j <slots>]\n", argv[0]);
 }
@@ -67,6 +73,14 @@ static int errfn(const char *fmt, va_list ap)
 	return vfprintf(stderr, fmt, ap);
 }
 
+static int openfn(const char *path, int flags,
+		  void *user_data __attribute__((unused)))
+{
+	printf("ts_print_mt: opening %s\n", path);
+
+	return open(path, flags);
+}
+
 int main(int argc, char **argv)
 {
 	struct tsdev *ts;
@@ -81,6 +95,7 @@ int main(int argc, char **argv)
 	int read_samples = 1;
 	short non_blocking = 0;
 	short raw = 0;
+	struct ts_lib_version_data *ver = ts_libversion();
 
 #ifndef TSLIB_VERSION_MT /* < 1.10 */
 	printf("You are running an old version of tslib. Please upgrade.\n");
@@ -142,12 +157,20 @@ int main(int argc, char **argv)
 		}
 
 		if (errno) {
-			char *str = "option ?";
+			char str[9];
+			sprintf(str, "option ?");
 			str[7] = c & 0xff;
 			perror(str);
 		}
 	}
 
+	ts_error_fn = errfn;
+
+#ifdef TSLIB_VERSION_OPEN_RESTRICTED
+	if (ver->features & TSLIB_VERSION_OPEN_RESTRICTED)
+		ts_open_restricted = openfn;
+#endif
+
 	if (non_blocking)
 		ts = ts_setup(tsdevice, 1);
 	else
@@ -158,8 +181,6 @@ int main(int argc, char **argv)
 		return errno;
 	}
 
-	ts_error_fn = errfn;
-
 #ifdef TS_HAVE_EVDEV
 	if (ioctl(ts_fd(ts), EVIOCGABS(ABS_MT_SLOT), &slot) < 0) {
 		perror("ioctl EVIOGABS");
@@ -205,7 +226,7 @@ int main(int argc, char **argv)
 
 		for (j = 0; j < ret; j++) {
 			for (i = 0; i < max_slots; i++) {
-				if (samp_mt[j][i].valid != 1)
+				if (!(samp_mt[j][i].valid & TSLIB_MT_VALID))
 					continue;
 
 				printf(YELLOW "sample %d - %ld.%06ld -" RESET " (slot %d) %6d %6d %6d\n",
