@@ -91,7 +91,13 @@ static void help(void)
 {
 	struct ts_lib_version_data *ver = ts_libversion();
 
-	printf("tslib %s (library 0x%X)\n", ver->package_version, ver->version_num);
+	printf("                 _       _ _ _\n");
+	printf("                | |_ ___| (_) |__\n");
+	printf("                | __/ __| | | '_ \\\n");
+	printf("                | |_\\__ \\ | | |_) |\n");
+	printf("                 \\__|___/_|_|_.__/\n\n");
+	printf("tslib %s / libts ABI version %d (0x%06X)\n",
+		ver->package_version, ver->version_num >> 16, ver->version_num);
 	printf("\n");
 	printf("Usage: ts_test_mt [-v] [-i <device>] [-j <slots>] [-r <rotate_value>]\n");
 	printf("\n");
@@ -183,7 +189,8 @@ int main(int argc, char **argv)
 		}
 
 		if (errno) {
-			char *str = "option ?";
+			char str[9];
+			sprintf(str, "option ?");
 			str[7] = c & 0xff;
 			perror(str);
 		}
@@ -300,7 +307,7 @@ int main(int argc, char **argv)
 			continue;
 
 		for (j = 0; j < max_slots; j++) {
-			if (samp_mt[0][j].valid != 1)
+			if (!(samp_mt[0][j].valid & TSLIB_MT_VALID))
 				continue;
 
 			for (i = 0; i < NR_BUTTONS; i++) {
