@@ -166,7 +166,7 @@ static int ts_verify_read_mt_1(struct ts_verify *data, int nr,
 
 		for (j = 0; j < ret; j++) {
 			for (i = 0; i < data->slots; i++) {
-				if (data->samp_mt[j][i].valid != 1)
+				if (!(data->samp_mt[j][i].valid & TSLIB_MT_VALID))
 					continue;
 
 				if (data->samp_mt[j][i].pressure > 255) {
@@ -485,14 +485,15 @@ int main(int argc, char **argv)
 		}
 
 		if (errno) {
-			char *str = "option ?";
+			char str[9];
+			sprintf(str, "option ?");
 			str[7] = c & 0xff;
 			perror(str);
 		}
 	}
 
 	unlink(CONFFILE);
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module_raw input\n");
 	fclose(data.tsconf);
 
@@ -500,14 +501,19 @@ int main(int argc, char **argv)
 		return errno;
 
 
-	printf("tslib %s (library 0x%X)\n", ver->package_version, ver->version_num);
-
+	printf("                 _       _ _ _\n");
+	printf("                | |_ ___| (_) |__\n");
+	printf("                | __/ __| | | '_ \\\n");
+	printf("                | |_\\__ \\ | | |_) |\n");
+	printf("                 \\__|___/_|_|_.__/\n\n");
+	printf("tslib %s / libts ABI version %d (0x%06X)\n",
+		ver->package_version, ver->version_num >> 16, ver->version_num);
 	printf("======================================================\n");
 	printf("input_raw\n");
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module skip nhead=1 ntail=1\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -515,7 +521,7 @@ int main(int argc, char **argv)
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module pthres pmin=20\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -523,7 +529,7 @@ int main(int argc, char **argv)
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module debounce drop_threshold=40\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -531,7 +537,7 @@ int main(int argc, char **argv)
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module median depth=7\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -539,7 +545,7 @@ int main(int argc, char **argv)
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module dejitter delta=100\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -547,7 +553,7 @@ int main(int argc, char **argv)
 
 	run_tests(&data);
 
-	data.tsconf = fopen(CONFFILE,"a+");
+	data.tsconf = fopen(CONFFILE, "a+");
 	fprintf(data.tsconf, "module linear\n");
 	fclose(data.tsconf);
 	printf("======================================================\n");
@@ -557,6 +563,8 @@ int main(int argc, char **argv)
 
 	unlink(CONFFILE);
 
+	printf("------------------------------------------------------\n");
+	printf("summary of results:\n");
 	printf("------------------------------------------------------\n");
 	printf("read_mt FAILs: %3d of %3d\n", data.read_mt_fail_count, data.read_mt_run_count);
 	printf("read    FAILs: %3d of %3d (WONTFIX)\n", data.read_fail_count, data.read_run_count);
