@@ -116,7 +116,13 @@ static void help(void)
 {
 	struct ts_lib_version_data *ver = ts_libversion();
 
-	printf("tslib %s (library 0x%X)\n", ver->package_version, ver->version_num);
+	printf("                 _       _ _ _\n");
+	printf("                | |_ ___| (_) |__\n");
+	printf("                | __/ __| | | '_ \\\n");
+	printf("                | |_\\__ \\ | | |_) |\n");
+	printf("                 \\__|___/_|_|_.__/\n\n");
+	printf("tslib %s / libts ABI version %d (0x%06X)\n",
+		ver->package_version, ver->version_num >> 16, ver->version_num);
 	printf("\n");
 	printf("Starts tslib instance listening to given event <device>, creates a virtual\n");
 	printf("input event device with given <name> using 'uinput', then continually reads\n");
@@ -158,7 +164,7 @@ static int send_touch_events(struct data_t *data, struct ts_sample_mt **s,
 				c++;
 			}
 
-			if (s[j][i].valid != 1)
+			if (!(s[j][i].valid & TSLIB_MT_VALID))
 				continue;
 
 			data->ev[c].time = s[j][i].tv;
@@ -440,7 +446,7 @@ static int process(struct data_t *data, struct ts_sample_mt **s_array,
 				       ": sample %d:  x\ty\tslot\ttracking_id\n"
 				       RESET, j);
 				for (i = 0; i < max_slots; i++) {
-					if (s_array[j][i].valid == 1) {
+					if (s_array[j][i].valid & TSLIB_MT_VALID) {
 						printf(DEFAULT_UINPUT_NAME
 						       ": \t%d\t%d\t%d\t%d\n",
 						       s_array[j][i].x,
@@ -603,8 +609,8 @@ int main(int argc, char **argv)
 		}
 
 		if (errno) {
-			char *str = "option ?";
-
+			char str[9];
+			sprintf(str, "option ?");
 			str[7] = c & 0xff;
 			perror(str);
 		}
